@@ -1,5 +1,5 @@
 CLI_CONFIG = {
-    "config": {"options": ["-c"], "help": "The path to a .yaml config file"},
+    "config": {"options": ["-c"], "subcommands": ["_global_"]},
     # Describe Options
     "desc_glob": {
         "display_priority": 0,
@@ -9,8 +9,7 @@ CLI_CONFIG = {
     },
     "progress": {
         "action": "store_true",
-        "subcommands": ["describe", "refresh"],
-        "help": "Show a progress bar",
+        "subcommands": ["describe", "refresh", "state"],
     },
     "hard_fail": {
         "action": "store_true",
@@ -48,8 +47,14 @@ CLI_CONFIG = {
         "subcommands": ["state", "validate"],
     },
     "cache_dir": {"subcommands": ["state", "validate", "refresh", "restore"]},
-    "root_dir": {"subcommands": ["state", "validate"]},
-    "params": {"subcommands": ["state", "describe"]},
+    "root_dir": {"subcommands": ["state", "validate", "refresh", "restore"]},
+    "params": {"nargs": "*", "subcommands": ["state", "describe"]},
+    "group": {"subcommands": ["state"]},
+    "invert": {
+        "action": "store_true",
+        "subcommands": ["state", "validate"],
+        "help": "Inverts the action taken during state execution. For example, absent in case of present and present in case of absent.",
+    },
     "render": {
         "subcommands": ["state", "validate"],
     },
@@ -66,9 +71,18 @@ CLI_CONFIG = {
     },
     "output": {
         "source": "rend",
-        "subcommands": ["exec", "state", "describe", "decrypt", "validate", "refresh"],
+        "subcommands": [
+            "exec",
+            "state",
+            "describe",
+            "decrypt",
+            "validate",
+            "refresh",
+            "doc",
+        ],
     },
     "sls": {"positional": True, "nargs": "*", "subcommands": ["state", "validate"]},
+    "target": {"subcommands": ["state"]},
     "esm_plugin": {"subcommands": ["state", "validate", "refresh", "restore"]},
     "esm_profile": {"subcommands": ["state", "validate", "refresh", "restore"]},
     # ACCT options
@@ -83,12 +97,10 @@ CLI_CONFIG = {
     },
     "acct_file": {
         "source": "acct",
-        "os": "ACCT_FILE",
         "subcommands": ["state", "exec", "describe", "validate", "refresh", "restore"],
     },
     "acct_key": {
         "source": "acct",
-        "os": "ACCT_KEY",
         "subcommands": [
             "state",
             "exec",
@@ -101,13 +113,24 @@ CLI_CONFIG = {
         ],
     },
     "acct_profile": {
-        "os": "ACCT_PROFILE",
         "subcommands": ["state", "exec", "describe", "validate", "refresh"],
     },
     "crypto_plugin": {
         "source": "acct",
         "subcommands": ["encrypt", "decrypt"],
     },
+    "render_pipe": {
+        "source": "acct",
+        "subcommands": ["encrypt", "decrypt"],
+    },
+    "extras": {
+        "source": "acct",
+        "subcommands": ["state", "exec", "describe", "validate", "refresh"],
+    },
+    "overrides": {
+        "source": "acct",
+        "subcommands": ["state", "exec", "describe", "validate", "refresh"],
+    },
     # EVBUS options
     "serialize_plugin": {
         "source": "evbus",
@@ -115,12 +138,21 @@ CLI_CONFIG = {
     },
     # RESTORE options
     "esm_cache_file": {"positional": True, "subcommands": ["restore"]},
+    # DOC options
+    "ref": {
+        "type": str,
+        "help": "The reference on the hub to show",
+        "source": "pop_tree",
+        "positional": True,
+        "subcommands": ["doc"],
+        "nargs": "?",
+    },
 }
 
 CONFIG = {
     "config": {
         "default": None,
-        "help": "Load extra options from a configuration file onto hub.OPT.idem",
+        "help": "Load extra options from a .yaml configuration file onto hub.OPT",
     },
     "esm_plugin": {
         "default": "local",
@@ -130,6 +162,10 @@ CONFIG = {
         "default": "msgpack",
         "help": "The serial plugin used to store ESM data",
     },
+    "esm_keep_cache": {
+        "default": False,
+        "help": "Do not delete the local ESM cache after it has been flushed to the ESM plugin",
+    },
     "esm_profile": {
         "default": "default",
         "help": "The acct profile to use with the enforced state manager",
@@ -167,7 +203,7 @@ CONFIG = {
         "help": 'The root directory to run idem from. By default it will be "/", or in the case of running as non-root it is set to <HOMEDIR>/.idem',
     },
     "render": {
-        "default": "jinja|yaml",
+        "default": "jinja|yaml|replacements",
         "help": "The render pipe to use, this allows for the language to be specified",
     },
     "runtime": {
@@ -192,13 +228,35 @@ CONFIG = {
         "default": "default",
     },
     "reconciler": {
-        "default": "none",
-        "help": "The reconciler plugin to use, 'none' by default",
+        "default": "basic",
+        "help": "The reconciler plugin to use, 'basic' by default",
     },
     "pending": {
         "default": "default",
         "help": "The pending plugin to use within the reconciler, 'default' by default",
     },
+    "progress": {
+        "default": False,
+        "help": "Show a progress bar",
+    },
+    "progress_options": {
+        "default": {},
+        "help": "Keyword options to pass to the tqdm progress bar constructor",
+        "type": dict,
+    },
+    "progress_plugin": {
+        "default": "tqdm",
+        "help": "The progress bar plugin to use",
+        "type": dict,
+    },
+    "group": {
+        "default": "number",
+        "help": "The group plugins, separated by pipes, to pass the data through before it gets rendered by outputters",
+    },
+    "target": {
+        "default": "",
+        "help": "Specify a single resource declaration ID. The resource and all its dependencies will be executed.",
+    },
 }
 SUBCOMMANDS = {
     "encrypt": {
@@ -235,9 +293,14 @@ SUBCOMMANDS = {
         "desc": "Restore the enforced managed state from a cache file",
         "help": "Restore the enforced managed state from a cache file",
     },
+    "doc": {
+        "desc": "Print documentation from code",
+        "help": "Print documentation from code",
+    },
 }
 DYNE = {
     "esm": ["esm"],
+    "source": ["source"],
     "idem": ["idem"],
     "exec": ["exec"],
     "log": ["log"],
@@ -246,4 +309,5 @@ DYNE = {
     "output": ["output"],
     "reconcile": ["reconcile"],
     "acct": ["acct"],
+    "group": ["group"],
 }
