@@ -18,7 +18,7 @@ async def _arun_catching_errors(hub, ctx):
 
 
 def _format_error(e: Exception):
-    return dict(comment=f"{e.__class__.__name__}: {e}", result=False, ret=None)
+    return dict(comment=[f"{e.__class__.__name__}: {e}"], result=False, ret=None)
 
 
 def call(hub, ctx):
