@@ -11,6 +11,9 @@ def __init__(hub):
 
 
 def ping(hub):
+    """
+    Immediately return success
+    """
     return {"result": True, "ret": True}
 
 
@@ -44,6 +47,17 @@ async def aecho(hub, value):
     return value
 
 
+def more(hub, ctx, *args, **kwargs):
+    """
+    Return the ctx and all parameters passed to this state.
+    """
+    return {
+        "result": True,
+        "comment": None,
+        "ret": {"args": args, "kwargs": kwargs, "ctx": ctx},
+    }
+
+
 async def event(
     hub,
     body: str,
