@@ -13,7 +13,8 @@ the lowstate
             "soft_require": [
                 "cheese.func_C",
             ],
-        }
+        },
+        "unique": ["present", "absent"],
     }
 """
 import pop.loader
@@ -53,17 +54,26 @@ def apply(hub, subs, low):
     for chunk in low:
         if not chunk["state"] in treq:
             continue
-        if not chunk["fun"] in treq[chunk["state"]]:
-            continue
-        rule = treq[chunk["state"]][chunk["fun"]]
-        for rule_name, rule_refs in rule.items():
-            for ref in rule_refs:
-                for req_chunk in low:
-                    req_path = f'{req_chunk["state"]}.{req_chunk["fun"]}'
-                    if req_path == ref:
-                        if rule_name not in chunk:
-                            chunk[rule_name] = []
-                        chunk[rule_name].append(
-                            {req_chunk["state"]: req_chunk["__id__"]}
-                        )
+        if chunk["fun"] in treq[chunk["state"]]:
+            rule = treq[chunk["state"]][chunk["fun"]]
+            _add_rule_to_chunk(low, chunk, rule)
+        if "unique" in treq[chunk["state"]]:
+            functions = treq[chunk["state"]]["unique"]
+            _add_unique_to_chunk(low, chunk, functions)
     return low
+
+
+def _add_rule_to_chunk(low, chunk, rule):
+    for rule_name, rule_refs in rule.items():
+        for ref in rule_refs:
+            for req_chunk in low:
+                req_path = f'{req_chunk["state"]}.{req_chunk["fun"]}'
+                if req_path == ref:
+                    if rule_name not in chunk:
+                        chunk[rule_name] = []
+                    chunk[rule_name].append({req_chunk["state"]: req_chunk["__id__"]})
+
+
+def _add_unique_to_chunk(low, chunk, functions):
+    if chunk["fun"] in functions:
+        chunk["unique"] = chunk["fun"]
