@@ -47,7 +47,13 @@ def high(hub, high):
                         # Add the requires to the reqs dict and check them
                         # all for recursive requisites.
                         argfirst = next(iter(arg))
-                        if argfirst in ("require", "watch", "prereq", "onchanges"):
+                        if argfirst in (
+                            "require",
+                            "watch",
+                            "prereq",
+                            "onchanges",
+                            "arg_bind",
+                        ):
                             if not isinstance(arg[argfirst], list):
                                 errors.append(
                                     f"The {argfirst}"
