@@ -1,4 +1,4 @@
-def stage(hub, name):
+def stage(hub, name: str):
     """
     Take the highdata and convert argument references to arg_bind requisites
     """
