@@ -1,5 +1,5 @@
 import asyncio
-import fnmatch
+import re
 import warnings
 from typing import Any
 from typing import Dict
@@ -8,7 +8,6 @@ import jmespath
 import pop.contract
 import pop.hub
 import pop.loader
-import tqdm
 
 
 async def run(
@@ -30,24 +29,23 @@ async def run(
     :param progress:
     :return:
     """
-    ctx = await hub.idem.acct.ctx(
-        path=desc_glob,
-        acct_file=acct_file,
-        acct_key=acct_key,
-        acct_profile=acct_profile,
-    )
     result = {}
-    coros = [_ async for _ in hub.idem.describe.recurse(ctx, hub.states, desc_glob)]
-    async for coro in hub.idem.describe.recurse(ctx, hub.exec, desc_glob):
+    coros = [
+        _
+        async for _ in hub.idem.describe.recurse(
+            hub.states, desc_glob, acct_file, acct_key, acct_profile
+        )
+    ]
+    async for coro in hub.idem.describe.recurse(
+        hub.exec, desc_glob, acct_file, acct_key, acct_profile
+    ):
         coros.append(coro)
 
-    progress_bar = None
-    if progress:
-        progress_bar = tqdm.tqdm(total=len(coros))
+    # Create the progress bar
+    progress_bar = hub.tool.progress.init.create(coros)
 
     for ret in asyncio.as_completed(coros):
-        if progress:
-            progress_bar.update(1)
+        hub.tool.progress.init.update(progress_bar)
         try:
             result.update(await ret)
         except Exception as e:
@@ -68,7 +66,13 @@ async def run(
 
 
 async def recurse(
-    hub, ctx, mod: pop.loader.LoadedMod or pop.hub.Sub, glob: str, ref: str = ""
+    hub,
+    mod: pop.loader.LoadedMod or pop.hub.Sub,
+    glob: str,
+    acct_file: str,
+    acct_key: str,
+    acct_profile: str,
+    ref: str = "",
 ):
     if hasattr(mod, "_subs"):
         for sub in mod._subs:
@@ -76,7 +80,9 @@ async def recurse(
                 r = ".".join([ref, sub])
             else:
                 r = sub
-            async for c in hub.idem.describe.recurse(ctx, mod[sub], glob, r):
+            async for c in hub.idem.describe.recurse(
+                mod[sub], glob, acct_file, acct_key, acct_profile, r
+            ):
                 yield c
     if hasattr(mod, "_loaded"):
         for loaded in mod._loaded:
@@ -84,18 +90,32 @@ async def recurse(
                 r = ".".join([ref, loaded])
             else:
                 r = loaded
-            async for c in hub.idem.describe.recurse(ctx, mod[loaded], glob, r):
+            async for c in hub.idem.describe.recurse(
+                mod[loaded], glob, acct_file, acct_key, acct_profile, r
+            ):
                 yield c
 
     # Only describe functions that implement the "describe" contract
     if hasattr(mod, "__contracts__"):
         if "auto_state" in mod.__contracts__:
             # Handle auto state describe functions
+            ctx = await hub.idem.acct.ctx(
+                path=glob,
+                acct_file=acct_file,
+                acct_key=acct_key,
+                acct_profile=acct_profile,
+            )
             ctx.exec_mod_ref = ref
             coro = hub.states.auto_state.describe(ctx)
             yield hub.pop.Loop.create_task(coro)
         elif "resource" in mod.__contracts__:
-            if fnmatch.fnmatch(ref, glob):
+            if re.fullmatch(glob, ref):
+                ctx = await hub.idem.acct.ctx(
+                    path=ref,
+                    acct_file=acct_file,
+                    acct_key=acct_key,
+                    acct_profile=acct_profile,
+                )
                 if isinstance(mod.describe, pop.contract.ContractedAsync):
                     coro = mod.describe(ctx)
                 else:
@@ -114,7 +134,13 @@ async def recurse(
             "__contracts__ = ['resource']",
             DeprecationWarning,
         )
-        if fnmatch.fnmatch(ref, glob):
+        if re.fullmatch(glob, ref):
+            ctx = await hub.idem.acct.ctx(
+                path=ref,
+                acct_file=acct_file,
+                acct_key=acct_key,
+                acct_profile=acct_profile,
+            )
             if isinstance(mod.describe, pop.contract.ContractedAsync):
                 coro = mod.describe(ctx)
             else:
