@@ -6,16 +6,13 @@ async def put(
     hub,
     body: Any,
     profile: str = "default",
-    routing_key: str = None,
     tags: Dict[str, Any] = None,
 ):
-    if routing_key is None:
-        routing_key = hub.OPT.idem.run_name
     if tags is None:
         tags = {}
 
     await hub.evbus.broker.put(
-        routing_key=routing_key, profile=profile, body=dict(tags=tags, message=body)
+        profile=profile, body=dict(tags=tags, message=body, run_name=hub.idem.RUN_NAME)
     )
 
 
@@ -23,14 +20,11 @@ def put_nowait(
     hub,
     body: Any,
     profile: str = "default",
-    routing_key: str = None,
     tags: Dict[str, Any] = None,
 ):
-    if routing_key is None:
-        routing_key = hub.OPT.idem.run_name
     if tags is None:
         tags = {}
 
     hub.evbus.broker.put_nowait(
-        routing_key=routing_key, profile=profile, body=dict(tags=tags, message=body)
+        profile=profile, body=dict(tags=tags, message=body, run_name=hub.idem.RUN_NAME)
     )
