@@ -1,3 +1,4 @@
+import datetime
 from typing import Any
 from typing import Dict
 
@@ -22,6 +23,8 @@ STATE_REQUISITE_KEYWORDS = frozenset(
         "require_any",
         "listen",
         "arg_bind",
+        "sensitive",
+        "ignore_changes",
     ]
 )
 STATE_REQUISITE_IN_KEYWORDS = frozenset(
@@ -116,9 +119,34 @@ async def run(hub, name, ctx, low, seq_comp, running, run_num, managed_state):
     chunk = seq_comp["chunk"]
     tag = hub.idem.tools.gen_tag(chunk)
     esm_tag = hub.idem.managed.gen_tag(chunk)
+    start_time = datetime.datetime.now()
     skip_check = ["resolver"]
     rdats = {}
-    errors = []
+    running[tag] = {
+        "tag": tag,
+        "name": chunk["name"],
+        "changes": {},
+        "new_state": None,
+        "old_state": None,
+        "comment": None,
+        "rerun_data": None,
+        "result": False,
+        "esm_tag": esm_tag,
+        "__run_num": run_num,
+        "start_time": str(start_time),
+        "total_seconds": 0,
+        "sls_meta": hub.idem.RUNS[name]["meta"],
+    }
+    ctx["tag"] = tag
+    # 'rerun_data' is used to pass data between state re-runs during reconciliation
+    if (
+        "running" in hub.idem.RUNS[name]
+        and tag in hub.idem.RUNS[name]["running"]
+        and "rerun_data" in hub.idem.RUNS[name]["running"][tag]
+    ):
+        ctx["rerun_data"] = hub.idem.RUNS[name]["running"][tag].get("rerun_data", None)
+
+    errors = seq_comp.get("errors", [])
     for reqret in seq_comp.get("reqrets", []):
         req = reqret["req"]
         if req not in rdats:
@@ -130,37 +158,61 @@ async def run(hub, name, ctx, low, seq_comp, running, run_num, managed_state):
             if hasattr(hub.idem.rules, rule):
                 rdat = hub.idem.rules[rule].check(name, ctx, rules[rule], reqret, chunk)
                 rdats[req].append(rdat)
-    errors = hub.idem.resolver.init.resolve(rdats)
-    if errors:
-        running[tag] = {
-            "name": chunk["name"],
-            "changes": {},
-            "comment": "\n".join(errors),
-            "result": False,
-            "__run_num": run_num,
-        }
-        return
+
+    # Get the function to be called as early as possible
     func = hub.idem.rules.init.get_func(name, chunk)
+    ref = None
+    if func:
+        ref = f"{func.ref}.{func.__name__}"
+
+    errors += hub.idem.resolver.init.resolve(rdats)
+    if errors:
+        running[tag]["comment"] = "\n".join(errors)
+        running[tag]["total_seconds"] = (
+            datetime.datetime.now() - start_time
+        ).total_seconds()
+        await hub.idem.event.put(
+            profile="idem-run",
+            body=running[tag],
+            tags={
+                "ref": ref,
+                "type": "state-result",
+                "acct_details": None,
+            },
+        )
+        return
     await hub.idem.event.put(
         profile="idem-chunk",
         body=chunk,
-        tags={"ref": "idem.rules.init.run", "type": "state-chunk"},
+        tags={
+            "ref": ref,
+            "type": "state-chunk",
+        },
     )
     if func is None:
-        running[tag] = {
-            "name": chunk["name"],
-            "changes": {},
-            "comment": f'Could not find function to enforce {chunk["state"]}.'
-            f"Please make sure that the corresponding plugin is loaded.",
-            "result": False,
-            "__run_num": run_num,
-        }
+        running[tag]["comment"] = (
+            f'Could not find function to enforce {chunk["state"]}. '
+            f"Please make sure that the corresponding plugin is loaded."
+        )
+        running[tag]["total_seconds"] = (
+            datetime.datetime.now() - start_time
+        ).total_seconds()
+        await hub.idem.event.put(
+            profile="idem-run",
+            body=running[tag],
+            tags={
+                "ref": ref,
+                "type": "state-result",
+                "acct_details": None,
+            },
+        )
         return
     chunk["ctx"] = ctx
     chunk = await hub.idem.mod.init.modify(name, chunk)
     call = hub.idem.tools.format_call(
-        func,
-        chunk,
+        fun=func,
+        data=chunk,
+        ignore_changes=chunk.get("ignore_changes"),
         expected_extra_kws=STATE_INTERNAL_KEYWORDS,
         enforced_state=managed_state.get(esm_tag) or managed_state.get(tag),
     )
@@ -172,17 +224,57 @@ async def run(hub, name, ctx, low, seq_comp, running, run_num, managed_state):
     # This is when the state is actually called
     ret = func(*call["args"], **call["kwargs"])
     ret = await hub.pop.loop.unwrap(ret)
-    # Only update the state if
 
-    if ret.get("result") is True:
-        managed_state[esm_tag] = ret.get("new_state")
+    # Update ESM under the following conditions:
+    # ESM will be updated when Idem ran with "refresh" subcommand (which implies the `test` flag).
+    # Or, when Idem state is not running with --test
+    # When the above rule is met, one of the following conditions need to be met:
+    # - "force_save" is True (This implies the plugin state has insisted Idem to save a resource state)
+    # - "result" is True (This implies the Idem state operation is successful)
+    # - "old_state" doesn't exist but "new_state" exists. This implies a creation operation during the
+    #    Idem state run. The "new_state" will be saved even if the operation failed ("result" is False).
+    if (
+        (getattr(hub, "SUBPARSER", None) == "refresh" or not ctx.get("test"))
+        and (
+            ret.get("result")
+            or ret.get("force_save", False)
+            or ((not ret.get("old_state")) and ret.get("new_state"))
+        )
+        and not chunk["ctx"].get("skip_esm", False)
+    ):
+        new_state = ret.get("new_state")
+        if new_state:
+            # Changes were made, update the cache
+            managed_state[esm_tag] = new_state
+        else:
+            # The successful new_state shows a deleted resource, remove it from the cache
+            managed_state.pop(esm_tag, None)
+    # Remove force_save from ret as it has been used and does not need to be kept in ret
+    ret.pop("force_save", None)
+
     for req, rlist in rdats.items():
         for rdat in rlist:
             if "post" in rdat:
                 ret = rdat["post"](*call["args"], **call["kwargs"])
                 ret = await hub.pop.loop.unwrap(ret)
-    ret["__run_num"] = run_num
-    running[tag] = ret
+
+    # Update the result for reconciliation
+    hub.idem.RUNS[name]["running"][tag].update(ret)
+
+    running[tag].update(ret)
+    running[tag]["total_seconds"] = (
+        datetime.datetime.now() - start_time
+    ).total_seconds()
+
+    await hub.idem.event.put(
+        profile="idem-run",
+        body=running[tag],
+        tags={
+            "ref": ref,
+            "type": "state-result",
+            "acct_details": chunk["ctx"].get("acct_details"),
+        },
+    )
 
 
 def check(
