@@ -1,8 +1,16 @@
+import copy
 from typing import Dict
 
 
 async def runtime(
-    hub, name: str, ctx: Dict, seq: Dict, low: Dict, running: Dict, managed_state: Dict
+    hub,
+    name: str,
+    ctx: Dict,
+    seq: Dict,
+    low: Dict,
+    running: Dict,
+    managed_state: Dict,
+    progress,
 ):
     """
     Execute the runtime in parallel mode
@@ -17,10 +25,12 @@ async def runtime(
         # Nothing can be run, we have hit recursive requisite,
         # or we are done
         pass
+
     for ind in inds:
         await hub.idem.rules.init.run(
             name,
-            ctx,
+            # every state run should use unique ctx so that ctx.acct retains correct information during whole state execution.
+            copy.deepcopy(ctx),
             low,
             seq[ind],
             running,
@@ -28,5 +38,4 @@ async def runtime(
             managed_state,
         )
         hub.idem.RUNS[name]["run_num"] += 1
-    if len(low) <= len(running):
-        return
+        hub.tool.progress.init.update(progress)
