@@ -50,13 +50,16 @@ async def create(
     acct_blob: bytes,
     managed_state: Dict[str, Any],
     param_sources: List[str],
+    acct_data: Dict[str, Any] = None,
+    invert_state: bool = False,
 ):
     """
     Create a new instance to execute against
     """
-    acct_data = await hub.idem.acct.data(
-        acct_key=acct_key, acct_file=acct_file, acct_blob=acct_blob
-    )
+    if acct_data is None:
+        acct_data = await hub.idem.acct.data(
+            acct_key=acct_key, acct_file=acct_file, acct_blob=acct_blob
+        )
 
     hub.idem.RUNS[name] = {
         "sls_sources": sls_sources,
@@ -70,10 +73,26 @@ async def create(
         "resolved": set(),
         "files": set(),
         "high": {},
-        # params_processing flag is used to indicate to resolve.py
-        # code that we only intend to parameter parsing
-        "params_processing": False,
-        "params": {},
+        # Jinja render uses the params python dict object to read the value corresponding to
+        # any parameter. When the parameter is not present params dict object's get() returns
+        # the None object. We need to return __PYTHON_NONE__ instead, so that it gets suitably
+        # converted into the python None object in the replacement renderer.
+        # Here we are using type library function to create a new type object, that has get()
+        # function which returns self[key] if parameter (key) is present, default value (if
+        # specified), and __PYTHON_NONE__ otherwise. Python dict is used as base class.
+        # Thus, for all parameters that are not present in dict, this get() lambda returns
+        # __PYTHON_NONE__.
+        "params": type(
+            "",
+            (dict,),
+            {
+                "get": lambda self, key, value=None: self[key]
+                if key in self
+                else value
+                if value is not None
+                else "__PYTHON_NONE__"
+            },
+        )(),
         "post_low": [],
         "errors": [],
         "iorder": 100000,
@@ -87,7 +106,7 @@ async def create(
         "acct_data": acct_data,
         "status": Status.CREATED,
         "managed_state": managed_state,
-        "processed_sls_sources": {},
+        "invert_state": invert_state,
     }
 
 
@@ -101,6 +120,7 @@ async def validate(
     cache_dir: str,
     sls: List[str],
     test: bool = False,
+    invert_state: bool = False,
     acct_file: str = None,
     acct_key: str = None,
     acct_profile: str = "default",
@@ -123,6 +143,7 @@ async def validate(
         acct_blob=acct_blob,
         managed_state=None,
         param_sources=None,
+        invert_state=invert_state,
     )
     # Get the sls file
     # render it
@@ -130,12 +151,15 @@ async def validate(
     # Run the low data using act/idem
     hub.idem.RUNS[name]["params"] = hub.tool.parameter.get_validate_params()
     # process sls-sources
-    hub.idem.sls.source.init.process(name)
     hub.idem.RUNS[name]["status"] = Status.GATHERING
-    await hub.idem.resolve.gather(name, *sls)
+    await hub.idem.sls_source.init.gather(name, *sls)
     if hub.idem.RUNS[name]["errors"]:
         hub.idem.RUNS[name]["status"] = Status.GATHER_ERROR
         return
+    if not hub.idem.RUNS[name].get("high"):
+        # If there is no high data after rendering, that means the rendered result is empty. So we return.
+        hub.idem.state.update_status(name, Status.FINISHED)
+        return
     hub.idem.RUNS[name]["status"] = Status.COMPILING
     await hub.idem.state.compile(name)
     if hub.idem.RUNS[name]["errors"]:
@@ -153,18 +177,23 @@ async def apply(
     subs: List[str],
     cache_dir: str,
     sls: List[str],
+    target: str = None,
     test: bool = False,
+    invert_state: bool = False,
     acct_file: str = None,
     acct_key: str = None,
     acct_profile: str = "default",
     acct_blob: bytes = None,
+    acct_data: Dict[str, Any] = None,
     managed_state: Dict[str, Any] = None,
-    param_sources: List[str] = [],
-    params_file: str = None,
+    param_sources: List[str] = None,
+    params: List[str] = None,
 ):
     """
     Run idem!
     """
+    if params is None:
+        params = []
     await hub.idem.state.create(
         name,
         sls_sources,
@@ -177,29 +206,31 @@ async def apply(
         acct_key=acct_key,
         acct_profile=acct_profile,
         acct_blob=acct_blob,
+        acct_data=acct_data,
         managed_state=managed_state,
-        param_sources=param_sources,
+        param_sources=param_sources or [],
+        invert_state=invert_state,
     )
 
-    if params_file is not None and len(params_file) > 0:
-        # Ensuring that only parameters are getting parsed.
-        hub.idem.RUNS[name]["params_processing"] = True
-        await hub.idem.resolve.gather(name, params_file)
-        # Restore the value to False so that for rest of the SLS are handled in normal manner.
-        hub.idem.RUNS[name]["params_processing"] = False
+    hub.idem.state.update_status(name, Status.GATHERING)
+    if params:
+        await hub.idem.sls_source.param.gather(name, *params)
         if hub.idem.RUNS[name]["errors"]:
+            hub.idem.state.update_status(name, Status.GATHER_ERROR)
             return
-    # process sls-sources
-    hub.idem.sls.source.init.process(name)
+
     # Get the sls file
     # render it
     # compile high data to "new" low data (bypass keyword issues)
-    # Run the low data using act/idem
-    hub.idem.state.update_status(name, Status.GATHERING)
-    await hub.idem.resolve.gather(name, *sls)
+    # Run the low data using acct/idem
+    await hub.idem.sls_source.init.gather(name, *sls)
     if hub.idem.RUNS[name]["errors"]:
         hub.idem.state.update_status(name, Status.GATHER_ERROR)
         return
+    if not hub.idem.RUNS[name].get("high"):
+        # If there is no high data after rendering, that means the rendered result is empty. So we return.
+        hub.idem.state.update_status(name, Status.FINISHED)
+        return
     hub.idem.state.update_status(name, Status.COMPILING)
     await hub.idem.state.compile(name)
     if hub.idem.RUNS[name]["errors"]:
@@ -207,10 +238,7 @@ async def apply(
         return
 
     hub.idem.state.update_status(name, Status.RUNNING)
-    try:
-        await hub.idem.run.init.start(name)
-    finally:
-        hub.idem.state.update_status(name, Status.FINISHED)
+    await hub.idem.run.init.start(name, target=target)
 
 
 async def compile_(hub, name):
@@ -275,7 +303,7 @@ async def batch(
     default_acct_profile: str = "default",
     name: str = None,
     crypto_plugin: str = "fernet",
-    reconcile_plugin: str = None,
+    reconcile_plugin: str = "basic",
     pending_plugin: str = None,
     esm_plugin: str = None,
     esm_profile: str = None,
@@ -293,8 +321,10 @@ async def batch(
     :param default_acct_profile: The acct profile to use for states that don't have a profile explicitly defined
     :param name: A unique identifier for this batch's run
     :param crypto_plugin: The plugin that should be used to decrypt encrypted profile blobs
-    :param reconcile_plugin: The plugin for reconciliation
+    :param reconcile_plugin: The plugin for reconciliation. Default: 'basic'
     :param pending_plugin: The plugin for determining state's 'pending' status for reconciliation
+    :param esm_plugin: The plugin for esm
+    :param esm_profile: The profile for esm
     """
     name = name or f"name_{uuid.uuid4()}"
     runtime = runtime or hub.OPT.idem.runtime
@@ -308,8 +338,7 @@ async def batch(
     acct_key = acct_key or hub.OPT.acct.acct_key
     acct_profile = default_acct_profile or hub.OPT.acct.get("acct_profile", "default")
 
-    # TODO the default reconcile will change to "basic" once reconciliation loop is enabled
-    reconcile_plugin = reconcile_plugin or hub.OPT.idem.get("reconciler", "none")
+    reconcile_plugin = reconcile_plugin or hub.OPT.idem.get("reconciler", "basic")
     pending_plugin = pending_plugin or hub.OPT.idem.get("pending", "default")
 
     if profiles:
@@ -353,19 +382,28 @@ async def batch(
         acct_blob=encrypted_profiles,
         serial_plugin=hub.OPT.idem.esm_serial_plugin,
     )
-    async with context_manager as state:
-        apply_kwargs["managed_state"] = state
-        await hub.idem.state.apply(**apply_kwargs)
+    try:
+        async with context_manager as state:
+            apply_kwargs["managed_state"] = state
+            await hub.idem.state.apply(**apply_kwargs)
 
-        if hub.idem.RUNS[name]["errors"]:
-            # pop is required to remove the run from memory
-            run_data = hub.idem.RUNS.pop(name)
-            return run_data["errors"]
+            if hub.idem.RUNS[name]["errors"]:
+                # pop is required to remove the run from memory
+                run_data = hub.idem.RUNS.pop(name)
+                return run_data["errors"]
 
-        # Reconciliation loop
-        await hub.reconcile.init.run(
-            plugin=reconcile_plugin, pending_plugin=pending_plugin, **apply_kwargs
-        )
+            # Reconciliation loop
+            if test is False:
+                await hub.reconcile.init.run(
+                    plugin=reconcile_plugin,
+                    pending_plugin=pending_plugin,
+                    **apply_kwargs,
+                )
+    except Exception as e:
+        display = hub.output.nested.display(str(e))
+        print(display)
+        # Return a non-zero error code
+        return 1
 
     run_data = hub.idem.RUNS.pop(name)
     return run_data["running"]
