@@ -1,24 +1,12 @@
 from typing import Any
 from typing import Dict
-from typing import List
 
 
 async def sig_loop(
     hub,
-    pending_plugin,
-    name,
-    sls_sources,
-    render,
-    runtime,
-    cache_dir,
-    sls,
-    test,
-    acct_file: str = None,
-    acct_key: str = None,
-    acct_profile: str = None,
-    acct_blob: str = None,
-    subs: List[str] = None,
-    managed_state: Dict[str, Any] = None,
+    pending_plugin: str,
+    name: str,
+    apply_kwargs: Dict[str, Any],
 ):
     """
     Validate the signature of the reconciliation loop function
