@@ -4,5 +4,5 @@ import pop.hub
 
 def start():
     hub = pop.hub.Hub()
-    hub.pop.sub.add("idem.idem")
+    hub.pop.sub.add(dyne_name="idem")
     hub.idem.init.cli()
