@@ -7,7 +7,7 @@ import asyncio
 def _format_error(ctx, e):
     return {
         "changes": {},
-        "comment": f"{e.__class__.__name__}: {e}",
+        "comment": [f"{e.__class__.__name__}: {e}"],
         "name": ctx.kwargs["name"],
         "result": False,
     }
