@@ -29,8 +29,11 @@ calls, e.g. running, calling, logging, output filtering etc.
       test.fail_with_changes:
         - name: foo
 """
+import copy
 import inspect
 import random
+import threading
+import time
 
 TREQ = {
     "treq": {
@@ -38,11 +41,53 @@ TREQ = {
             "test.nop",
         ]
     },
+    "unique": ["unique_op"],
 }
 
+UNIQUE_LOCK = threading.Lock()
+
 __contracts__ = ["resource"]
 
 
+def __init__(hub):
+    hub.states.test.ACCT = ["test"]
+
+
+def acct(hub, ctx, name):
+    """
+    Return exactly what was passed into ctx.acct
+    """
+    ret = {
+        "name": name,
+        "result": True,
+        "comment": None,
+        "old_state": None,
+        "new_state": ctx.acct,
+    }
+    if not ctx.acct:
+        ret["result"] = False
+
+    return ret
+
+
+def parallel_state(hub, ctx, name):
+    """
+    Ensure that a ctx has unique values corresponding to state applied
+    """
+    ctx.acct = {"name": name}
+    # using sleep() to hault the code execution inorder to simulate the
+    # parallel execution issue(when first state sleeps, second state will run).
+    time.sleep(1)
+    ret = {
+        "name": name,
+        "result": True,
+        "comment": None,
+        "old_state": None,
+        "new_state": ctx,
+    }
+    return ret
+
+
 def treq(hub, ctx, name, **kwargs):
     """
     Ensure that a transparent requisite is applied
@@ -72,6 +117,21 @@ async def anop(hub, ctx, name, **kwargs):
     return succeed_without_changes(hub, ctx, name)
 
 
+async def unique_op(hub, ctx, name, **kwargs):
+    """
+    A state that is defined as 'unique' treq, and require a single instance
+    to be invoked at a time. performs locking.
+    """
+    # Throw an error if could not acquire the lock
+    assert UNIQUE_LOCK.acquire(blocking=False) == True, "unique_op is running. Locked!"
+    await hub.pop.loop.sleep(2)
+    UNIQUE_LOCK.release()
+
+    if ctx["test"]:
+        return none_without_changes(hub, ctx, name)
+    return succeed_without_changes(hub, ctx, name)
+
+
 def succeed_with_comment(hub, ctx, name, comment, **kwargs):
     ret = {
         "name": name,
@@ -257,19 +317,33 @@ def configurable_test_state(
     return ret
 
 
-def present(hub, ctx, name: str, new_state=None, result=True):
+def present(
+    hub,
+    ctx,
+    name: str,
+    old_state=None,
+    changes=None,
+    new_state=None,
+    result=True,
+    force_save=None,
+):
     """
-    Return the previous old_state and the given new_state.
+    Return the previous old_state, if it's not specified in sls, and the given new_state.
     Raise an error on fail
     """
-    old_state = ctx.get("old_state")
-    return {
+    if old_state is None:
+        old_state = ctx.get("old_state")
+    ret = {
         "name": name,
         "old_state": old_state,
         "new_state": new_state,
+        "changes": changes,
         "result": result,
         "comment": None,
     }
+    if force_save is not None:
+        ret["force_save"] = force_save
+    return ret
 
 
 def absent(hub, ctx, name: str, new_state=None, result=True):
@@ -290,7 +364,7 @@ async def describe(hub, ctx):
     ret = {}
     for func in hub.states.test:
         name = func.__name__
-        if name in ("present", "absent", "succeed_with_arg_bind"):
+        if name in ("present", "absent", "succeed_with_arg_bind", "describe"):
             continue
         ref = f"test.{name}"
         state_name = f"Description of {ref}"
@@ -309,3 +383,54 @@ async def describe(hub, ctx):
                     ret[state_name][ref].append({arg: p.default})
 
     return ret
+
+
+def update(hub, ctx, name, param_1, param_2=None, result=True):
+    old_state = ctx.get("old_state")
+    new_state = copy.deepcopy(old_state)
+    if new_state is None:
+        new_state = {}
+    new_state["param_1"] = param_1
+    # Only update param_2 when param_2 is not None
+    if param_2 is not None:
+        new_state["param_2"] = param_2
+    ret = {
+        "name": name,
+        "old_state": old_state,
+        "new_state": new_state,
+        "result": result,
+        "comment": None,
+    }
+    return ret
+
+
+def present_resource(
+    hub,
+    ctx,
+    name: str,
+    enforce_state=None,
+    remote_state=None,
+    resource_id=None,
+    result=True,
+):
+    """
+    This function simulates a present() function following a resource contract.
+    Given a enforce_state and a remote_state, apply the enforce_state to remote_state
+    """
+    if enforce_state is None:
+        enforce_state = {}
+    if remote_state is None:
+        remote_state = {}
+    for parameter, value in enforce_state.items():
+        # Update remote_state with enforce_state values if the property is not None.
+        # This is to simulate Idem's behavior of not changing a parameter when it is None.
+        if enforce_state[parameter] is not None:
+            remote_state[parameter] = enforce_state[parameter]
+    ret = {
+        "name": name,
+        "old_state": ctx.get("old_state"),
+        "new_state": remote_state,
+        "result": result,
+        "comment": None,
+    }
+    return ret
